"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class BitBuffer {
    get buffer() {
        return this._buffer;
    }
    set buffer(value) {
        this._buffer = value;
        this.length = value.length;
        if (value.length < BitBuffer.MaxHalfSize || value.length % 2 !== 0)
            return;
        this.left = new BitBuffer(this._buffer.slice(0, this.length / 2));
        this.right = new BitBuffer(this._buffer.slice(this.length / 2, this.length));
    }
    get left() {
        return this._left;
    }
    set left(value) {
        this._left = value;
        // add updated left to main buffer
        if (this.length < BitBuffer.MaxHalfSize)
            return;
        if (value) {
            for (let i = 0; i < value.length; i++) {
                this.buffer[i] = value.buffer[i];
            }
        }
    }
    get right() {
        return this._right;
    }
    set right(value) {
        this._right = value;
        if (this.length < BitBuffer.MaxHalfSize)
            return;
        if (value) {
            for (let i = 0; i < value.length; i++) {
                this.buffer[i + value.length] = value.buffer[i];
            }
        }
    }
    constructor(param1, param2) {
        this._left = undefined;
        this._right = undefined;
        this._buffer = [];
        this.length = 0;
        if (Array.isArray(param1)) {
            if (param2 === undefined) {
                this.length = param1.length;
                this.buffer = param1;
            }
            else {
                // check both left and right for equal length
                if (param1.length !== param2.length)
                    throw new Error("Left and right halves must be of equal length");
                // add length
                this.length = param1.length + param2.length;
                // fill main buffer with concat values
                this.buffer = [...param1, ...param2];
                // fill right and left halves
                this.left = new BitBuffer(param1);
                this.right = new BitBuffer(param2);
            }
        }
        else {
            BitBuffer.checkLength(param1);
            this.length = param1;
            this.buffer = new Array(param1).fill(0);
        }
    }
    // a function for limit length inputs of the class
    static checkLength(length) {
        // Check if the length is valid
        const maxLength = 10;
        if (length > maxLength)
            throw new Error("Length must be less than " + maxLength);
    }
    // a function for left shifting by N
    shiftLeft(child, position) {
        // Check if the shift is valid
        if (position > child.length)
            throw new Error("Shift must be less than " + child.length);
        // Shift the buffer
        const newBuffer = new Array(child.length);
        for (let i = 0; i < child.length; i++) {
            let index = i + position;
            if (index >= child.length)
                index -= child.length;
            newBuffer[i] = child.buffer[index];
        }
        return new BitBuffer(newBuffer);
    }
    leftShiftLeft(position) {
        if (!this.left)
            throw new Error(`Left Buffer is ${this.left}`);
        this.left = this.shiftLeft(this.left, position);
    }
    leftShiftRight(position) {
        if (!this.right)
            throw new Error(`Right Buffer is ${this.right}`);
        this.right = this.shiftLeft(this.right, position);
    }
    // Swap left part and right part of buffer
    swap() {
        [this.left, this.right] = [this.right, this.left];
        return this;
    }
    // xor all bits of two buffers
    xor(second) {
        const newBuffer = new Array(this.length);
        for (let i = 0; i < this.length; i++)
            // tslint:disable-next-line:no-bitwise
            newBuffer[i] = this.buffer[i] ^ second.buffer[i];
        return new BitBuffer(newBuffer);
    }
    // stringify the Buffer part of the class
    toString() {
        let stringed = "";
        for (let i = 0; i < this.length; i++)
            stringed += `${this.buffer[i]}`;
        return stringed;
    }
}
/**
 * BitBuffer class is the core class for computational process,
 * shifting.
 *
 * @param MaxHalfSize - Half the size that `BitBuffer` can store
 * @param _buffer - Private property that the main data will be stored in.
 * @param _left - Left part of `_buffer`, type of this property is `BitBuffer`
 * and the size is half of the `_buffer`.
 * @param _right - Right part of `_buffer`, type of this property is `BitBuffer`
 * and the size is half of the `_buffer`.
 */
BitBuffer.MaxHalfSize = 5;
exports.default = BitBuffer;
